@@ -5,7 +5,11 @@ use Carp;
 use Cwd;
 use IPC::Open2;
 
-our $VERSION = '0.04';
+use Git::FastExport::Block;
+
+our $VERSION = '0.05';
+
+'progress 1 objects';
 
 sub new {
     my ( $class, $repo ) = @_;
@@ -91,38 +95,6 @@ sub next_block {
     return $block;
 }
 
-package Git::FastExport::Block;
-
-my $LF = "\012";
-
-my %fields = (
-    commit     => [qw( mark author committer data from merge files )],
-    tag        => [qw( from tagger data )],
-    reset      => [qw( from )],
-    blob       => [qw( mark data )],
-    checkpoint => [],
-    progress   => [],
-);
-
-sub as_string {
-    my ($self) = @_;
-    my $string = $self->{header} . $LF;
-
-    for my $key ( @{ $fields{ $self->{type} } } ) {
-        next if !exists $self->{$key};
-        if ( $key eq 'data' ) {
-            $string
-                .= 'data ' . length( $self->{data} ) . $LF . $self->{data};
-        }
-        else {
-            $string .= "$_$LF" for @{ $self->{$key} };
-        }
-    }
-    return $string .= $self->{footer} || '';
-}
-
-1;
-
 __END__
 
 =head1 NAME
